"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var chai_1 = require("chai");
describe(__filename, function () {
    [
        [
            "Can render changes from two different fiefiles",
            {
                "entry.pc": "A"
            },
            {
                "entry.pc": "B"
            }
        ],
        [
            "Properly replaces style rules",
            {
                "entry.pc": "<style>.a {color: red;  } </style>a"
            },
            {
                "entry.pc": "<style>.b {color: red;  } </style>a"
            }
        ],
        [
            "Properly removes style rules",
            {
                "entry.pc": "<style>.a {color: red;  } .b {color: red; } </style>a"
            },
            {
                "entry.pc": "<style>.a { color: red; } </style>a"
            }
        ],
        [
            "Inserts new rules",
            {
                "entry.pc": "<style>.a { color: red; } </style>a"
            },
            {
                "entry.pc": "<style>.a { color: red; } .b { color: blue; } </style>a"
            }
        ],
        [
            "Updates CSS from module",
            {
                "entry.pc": "<import src=\"./module.pc\" as=\"module\" /><module.Test />",
                "module.pc": "<div export component as=\"Test\"><style>color: before;</style></div>"
            },
            {
                "module.pc": "<div export component as=\"Test\"><style>color: after;</style></div>"
            }
        ],
        [
            "Updates CSS from module _module_",
            {
                "entry.pc": "<import src=\"./a.pc\" as=\"module\" /><module.Test />",
                "a.pc": "<import src=\"./b.pc\" as=\"module\" />\n          <module.Test export component as=\"Test\">\n            <style>color: red;</style>\n          </module.Test>",
                "b.pc": "<div export component as=\"Test\">\n          <style>background: green;</style>\n        </div>"
            },
            {
                "b.pc": "<div export component as=\"Test\">\n          <style>background: orange;</style>\n        </div>"
            }
        ],
        [
            "Can add new frams & still maintain styles",
            {
                "entry.pc": "\n          <div>\n            <style>\n              color: red;\n            </style>\n            Test\n          </div>\n        "
            },
            {
                "entry.pc": "\n          <div>\n            <style>\n              color: blue;\n            </style>\n            Test A\n          </div>\n          <div>\n            <style>\n              color: red;\n            </style>\n            Test B\n          </div>\n        "
            }
        ],
        [
            "Properly patches between",
            {
                "entry.pc": "\n          <span />\n        "
            },
            {
                "entry.pc": "\n          <style>\n            .dbe {\n              bacc: qr;\n            }\n          </style>\n        "
            },
            {
                "entry.pc": "\n          <span />\n        "
            }
        ],
        [
            "Maintains frame with previous sibling is hidden",
            {
                "entry.pc": "\n          <a />\n          <b />\n          <c />\n        "
            },
            {
                "entry.pc": "\n          <!-- \n            @frame { visible: false }\n          -->\n          <a />\n          <b />\n          <c />\n        "
            }
        ],
        [
            "Can handle charset",
            {
                "entry.pc": "\n          <style>\n            @charset \"utf-8\";\n            div {\n              color: red;\n            }\n          </style>\n        "
            },
            {
                "entry.pc": "\n          <style>\n            div {\n              color: red;\n            }\n          </style>\n        "
            }
        ],
        [
            "Can handle charset",
            {
                "entry.pc": "\n          <div data />\n        "
            },
            {
                "entry.pc": "\n          <div data-l />\n        "
            }
        ]
    ].forEach(function (_a) {
        var title = _a[0], initial = _a[1], changes = _a.slice(2);
        it(title, function () { return __awaiter(void 0, void 0, void 0, function () {
            var engine, renderer, _a, _b, _i, changes_1, change, _c, _d, _e, name_1, baseline, _f, _g;
            return __generator(this, function (_h) {
                switch (_h.label) {
                    case 0:
                        engine = (0, utils_1.createMockEngine)(initial);
                        renderer = (0, utils_1.createMockFramesRenderer)("entry.pc");
                        engine.onEvent(renderer.handleEngineDelegateEvent);
                        _b = (_a = renderer).initialize;
                        return [4 /*yield*/, engine.open("entry.pc")];
                    case 1:
                        _b.apply(_a, [(_h.sent())]);
                        _i = 0, changes_1 = changes;
                        _h.label = 2;
                    case 2:
                        if (!(_i < changes_1.length)) return [3 /*break*/, 10];
                        change = changes_1[_i];
                        _c = [];
                        for (_d in change)
                            _c.push(_d);
                        _e = 0;
                        _h.label = 3;
                    case 3:
                        if (!(_e < _c.length)) return [3 /*break*/, 6];
                        name_1 = _c[_e];
                        return [4 /*yield*/, engine.updateVirtualFileContent(name_1, change[name_1])];
                    case 4:
                        _h.sent();
                        _h.label = 5;
                    case 5:
                        _e++;
                        return [3 /*break*/, 3];
                    case 6:
                        baseline = (0, utils_1.createMockFramesRenderer)("entry.pc");
                        _g = (_f = baseline).initialize;
                        return [4 /*yield*/, engine.open("entry.pc")];
                    case 7: return [4 /*yield*/, _g.apply(_f, [(_h.sent())])];
                    case 8:
                        _h.sent();
                        (0, chai_1.expect)(combineFrameHTML(renderer)).to.eql(combineFrameHTML(baseline));
                        _h.label = 9;
                    case 9:
                        _i++;
                        return [3 /*break*/, 2];
                    case 10: return [2 /*return*/];
                }
            });
        }); });
    });
    var combineFrameHTML = function (renderer) {
        return renderer
            .getState()
            .frames.map(function (frame) { return frame.stage.innerHTML; })
            .join("");
    };
});
